#!/bin/sh

MAP=m0
SIZE=102400
FAILDEV=d0
TYPE=mpath
#TYPE=mirror

echo "### Clean up (Errors are ignorable) ###"
dmsetup remove ${MAP}
echo "### End ###"

echo "### Set up DM dev ###"
dmsetup table $FAILDEV || exit 1

if [ "$TYPE" = "mpath" ]; then
	# multipath sample
	PV1=d00
	mpath_templ="multipath 1 queue_if_no_path 0 1 1 round-robin 0 1 1"
	mpath_rr="1000"
	map1="0 $SIZE $mpath_templ /dev/mapper/${FAILDEV} $mpath_rr"
	map2="0 $SIZE $mpath_templ /dev/mapper/${PV1} $mpath_rr"
	dmsetup remove ${PV1}
	dmsetup table ${FAILDEV} | dmsetup create ${PV1}
else
	# mirror sample
	PV1=d1
	PV2=d2
	map1="0 $SIZE mirror disk 3 /dev/mapper/${FAILDEV} 1024 block_on_error 2 /dev/mapper/${PV1} 0 /dev/mapper/${PV2} 0"
	map2="0 $SIZE mirror core 1 1024 2 /dev/mapper/${PV1} 0 /dev/mapper/${PV2} 0"
	dmsetup table $PV1 || exit 1
	dmsetup table $PV2 || exit 1
fi


orig_tab=$(dmsetup table ${FAILDEV})
echo $map1 | dmsetup create ${MAP}
dmsetup table ${MAP}
# prepare next tables
echo $map2 | dmsetup load ${MAP}
echo "0 $SIZE error" | dmsetup load ${FAILDEV}

echo "### Test I/O ###"
./do_fsync /dev/mapper/${MAP} ${FAILDEV} &
sleep 10
sync &

echo "### Suspend ###"
dmsetup status --noopencount ${MAP}
dmsetup suspend --noflush ${MAP}
dmsetup info --noopencount ${MAP}

echo "### Resume ###"
dmsetup resume ${MAP}

echo "### Wait and recover failed device ###"
sleep 10
echo "$orig_tab" | dmsetup load ${FAILDEV}
dmsetup resume ${FAILDEV}
